package com.cmeeting.thread.monitor;

import com.cmeeting.util.R;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import java.util.HashMap;
import java.util.Map;

@RestController
@RequestMapping("/threadPool")
public class ThreadPoolController {

    private final ThreadPoolMonitorService monitorService;

    @Autowired
    public ThreadPoolController(ThreadPoolMonitorService monitorService) {
        this.monitorService = monitorService;
    }

    // 获取所有线程池状态（中文键名）
    @GetMapping("/status")
    public R getAllStatusInChinese() {
        Map<String, ThreadPoolStatus> statusMap = new HashMap<>();

        monitorService.getAllExecutors().forEach((name, executor) -> {
            ThreadPoolStatus status = ThreadPoolStatus.fromExecutor(executor, name);
            if (status != null) {
                statusMap.put(name, status);
            }
        });

        return R.ok(statusMap);
    }

    // 获取指定线程池状态（中文键名）
    @GetMapping("/status/{executorName}")
    public R getStatusInChinese(@PathVariable String executorName) {
        ThreadPoolTaskExecutor executor = monitorService.getExecutor(executorName);

        if (executor == null) {
            return R.error();
        }

        ThreadPoolStatus status = ThreadPoolStatus.fromExecutor(executor, executorName);
        return R.ok(status);
    }
}
