package com.cmeeting.thread.monitor;

import com.fasterxml.jackson.databind.annotation.JsonSerialize;
import lombok.AllArgsConstructor;
import lombok.Data;

import java.text.DecimalFormat;

@Data
@AllArgsConstructor
@JsonSerialize(using = ThreadPoolStatusInfoSerializer.class)
public class ThreadPoolStatus {
    private String poolName;
    private Integer corePoolSize;
    private Integer maxPoolSize;
    private Integer currentPoolSize;
    private Integer activeThreadCount;
    private Integer queueSize;
    private Integer queueCapacity;
    private Long totalTasks;
    private Long completedTasks;
    private Long pendingTasks;
    private String threadUtilization;
    private String queueUtilization;
    private String runningStatus;
    private String healthStatus;

    // 从原始ThreadPoolTaskExecutor状态构建
    public static ThreadPoolStatus fromExecutor(
            org.springframework.scheduling.concurrent.ThreadPoolTaskExecutor executor,
            String poolName) {

        java.util.concurrent.ThreadPoolExecutor threadPoolExecutor = executor.getThreadPoolExecutor();
        if (threadPoolExecutor == null) {
            return null;
        }

        DecimalFormat df = new DecimalFormat("0.00%");
        String runningStatus = threadPoolExecutor.isShutdown() ?
                (threadPoolExecutor.isTerminated() ? "已终止" : "关闭中") : "运行中";

        int queueCapacity = 1000; // 从配置中获取或硬编码
        double queueUtilization = (double) threadPoolExecutor.getQueue().size() / queueCapacity;

        String healthStatus = threadPoolExecutor.getActiveCount() < executor.getMaxPoolSize() * 0.8 &&
                queueUtilization < 0.7 ? "健康" : "需要关注";

        return new ThreadPoolStatus(
                poolName,
                executor.getCorePoolSize(),
                executor.getMaxPoolSize(),
                threadPoolExecutor.getPoolSize(),
                threadPoolExecutor.getActiveCount(),
                threadPoolExecutor.getQueue().size(),
                queueCapacity,
                threadPoolExecutor.getTaskCount(),
                threadPoolExecutor.getCompletedTaskCount(),
                threadPoolExecutor.getTaskCount() - threadPoolExecutor.getCompletedTaskCount(),
                df.format((double) threadPoolExecutor.getActiveCount() / executor.getMaxPoolSize()),
                df.format(queueUtilization),
                runningStatus,
                healthStatus
        );
    }
}
